@@ -31,7 +31,6 @@
 #include "messages.h"
 #include "next_image.h"
 #include "mnemonics.h"
-#include "files_list.h"
 #include "loading.h"
 #include "thumbnails.h"
 #include "callbacks.h"
@@ -39,6 +38,7 @@
 #include "timestamp.h"
 #include "dirty_gtk.h"
 #include "gliv-image.h"
+#include "tree_browser.h"
 
 extern options_struct *options;
 extern GlivImage *current_image;
@@ -49,18 +49,16 @@ static GtkMenuItem *rebuilding_entry;
 /* The menu entry to cancel the rebuilding. */
 static GtkMenuItem *cancel_menu_item;
 
-/*
- * The number of files in the current tree.
- * Used also to know if we are rebuilding a menu.
- */
-static gint tree_count;
-
 /*
  * Association between a directory and its GtkMenu.
  * Used by Directories->"Current image directory...".
  */
 static GHashTable *dir_menu_hash = NULL;
 
+/* Menus timestamps */
+static DECLARE_TIMESTAMP(directories_timestamp);
+static DECLARE_TIMESTAMP(images_timestamp);
+
 /*** Functions for both menus. ***/
 
 /* Called at menu creation time. */
@@ -70,7 +68,7 @@ void set_rebuilding_entry(GtkMenuItem * item)
 }
 
 /* Refresh the percent indicator. */
-static void set_menu_indicator(gchar * name, gint percent)
+static void set_menu_indicator(const gchar * name, gint percent)
 {
     static gchar *rebuilding = NULL;
 
@@ -91,10 +89,25 @@ static void set_menu_indicator(gchar * name, gint percent)
 }
 
 /* Update the percent indicator. */
-static void set_progress(gchar * menu, gint * percent, gint number)
+void set_progress(const gchar * menu, gint * percent, gint number)
 {
-    if ((*percent + 1) * tree_count <= 100 * number) {
-        *percent = (100 * number) / tree_count;
+    static gint total_length;
+
+    /* Yes, that's ugly */
+    if (number == -1) {
+        /* First time for the menu */
+        total_length = tree_count_files();
+        return;
+    }
+
+    if (menu == NULL) {
+        /* Last time for the menu */
+        set_menu_indicator(NULL, 0);
+        return;
+    }
+
+    if ((*percent + 1) * total_length <= 100 * number) {
+        *percent = (100 * number) / total_length;
         set_menu_indicator(menu, *percent);
     }
 }
@@ -184,7 +197,6 @@ static void add_file_from_tree(DirtyGtkMenu * menu, tree_item * item)
 {
     static gchar *directories = NULL;
     static gint number = 0, percent = 0;
-    gchar *dirname;
 
     if (canceled_using_tree())
         return;
@@ -201,9 +213,6 @@ static void add_file_from_tree(DirtyGtkMenu * menu, tree_item * item)
         return;
     }
 
-    dirname = g_path_get_dirname(item->path);
-    g_hash_table_insert(dir_menu_hash, dirname,
-                        dirty_gtk_menu_get_tearoff(menu));
     add_menu_item(menu, item, MENU_FILE);
 
     set_progress(directories, &percent, number);
@@ -220,6 +229,9 @@ static DirtyGtkMenu *add_sub_menu(DirtyGtkMenu * parent, tree_item * item)
     menu = dirty_gtk_menu_new();
     gtk_menu_item_set_submenu(menu_item, GTK_WIDGET(menu->gtk_menu));
 
+    g_hash_table_insert(dir_menu_hash, item->path,
+                        dirty_gtk_menu_get_tearoff(menu));
+
     return menu;
 }
 
@@ -250,14 +262,27 @@ static void make_menu_from_tree_rec(GNode * tree, DirtyGtkMenu * parent)
 
 static gboolean open_current_dir_menu(void)
 {
-    if (dir_menu_hash && current_image) {
-        gchar *dir = g_path_get_dirname(current_image->node->data);
-        GtkTearoffMenuItem *tearoff = g_hash_table_lookup(dir_menu_hash, dir);
+    if (dir_menu_hash && current_image && current_image->node) {
+        gchar *path = g_strdup(current_image->node->data);
+        GtkTearoffMenuItem *tearoff;
 
-        g_free(dir);
+        do {
+            gchar *new_path = g_path_get_dirname(path);
 
-        if (tearoff)
+            g_free(path);
+            path = new_path;
+
+            tearoff = g_hash_table_lookup(dir_menu_hash, path);
+        } while (!tearoff && !g_str_equal(path, "/") &&
+                 !g_str_equal(path, "."));
+
+        g_free(path);
+
+        if (tearoff) {
+            GtkWidget *parent_menu = gtk_widget_get_parent(GTK_WIDGET(tearoff));
+            gtk_widget_realize(parent_menu);
             gtk_menu_item_activate(GTK_MENU_ITEM(tearoff));
+        }
     }
 
     return FALSE;
@@ -280,8 +305,10 @@ static void add_current_dir_entry(DirtyGtkMenu * menu)
     if (dir_menu_hash)
         g_hash_table_destroy(dir_menu_hash);
 
-    dir_menu_hash =
-        g_hash_table_new_full(g_str_hash, g_str_equal, g_free, NULL);
+    dir_menu_hash = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, NULL);
+
+    g_hash_table_insert(dir_menu_hash, ".", dirty_gtk_menu_get_tearoff(menu));
+    g_hash_table_insert(dir_menu_hash, "/", dirty_gtk_menu_get_tearoff(menu));
 }
 
 /*** Images menu. ***/
@@ -325,7 +352,7 @@ static gboolean add_file_item(GNode * tree, DirtyGtkMenu * parent_menu)
 static DirtyGtkMenu *begin_rebuild(GtkMenuItem * root, GCallback func)
 {
     static DirtyGtkMenu *menu;
-    GtkWidget *rebuilder;
+    GtkWidget *rebuilder, *browser;
 
     if (root != NULL)
         gtk_menu_item_deselect(root);
@@ -338,12 +365,19 @@ static DirtyGtkMenu *begin_rebuild(GtkMenuItem * root, GCallback func)
     g_signal_connect_swapped(rebuilder, "activate", func, NULL);
     dirty_gtk_menu_append(menu, rebuilder);
 
+    browser =
+        gtk_menu_item_new_with_mnemonic(add_mnemonic
+                                        (_("Open thumbnails browser...")));
+
+    g_signal_connect(browser, "activate", show_tree_browser, NULL);
+    dirty_gtk_menu_append(menu, browser);
+
     gtk_menu_item_set_submenu(root, GTK_WIDGET(menu->gtk_menu));
 
     return menu;
 }
 
-static GNode *get_tree(void)
+GNode *get_tree(void)
 {
     if (currently_loading())
         /*
@@ -355,7 +389,7 @@ static GNode *get_tree(void)
     return make_tree();
 }
 
-#define REBUILD_START(name, func)                                            \
+#define REBUILD_START(name, func, timestamp)                                 \
     do {                                                                     \
         if (root != NULL) {                                                  \
             menu_item = root;                                                \
@@ -370,11 +404,11 @@ static GNode *get_tree(void)
         if (tree == NULL)                                                    \
             return FALSE;                                                    \
                                                                              \
-        tree_count = tree_count_files(tree);                                 \
         gtk_widget_set_sensitive(GTK_WIDGET(cancel_menu_item), TRUE);        \
         reset_mnemonics();                                                   \
         menu = begin_rebuild(menu_item, G_CALLBACK(func));                   \
         set_menu_indicator(name, 0);                                         \
+        set_progress(NULL, NULL, -1);                                        \
     } while (0)
 
 static gboolean rebuild_end(GtkMenuItem * root, timestamp_t * ts,
@@ -385,7 +419,7 @@ static gboolean rebuild_end(GtkMenuItem * root, timestamp_t * ts,
 
     dirty_gtk_menu_append(menu, gtk_tearoff_menu_item_new());
 
-    set_menu_indicator(NULL, 0);
+    set_progress(NULL, NULL, 0);
     gtk_widget_set_sensitive(GTK_WIDGET(cancel_menu_item), FALSE);
 
     if (canceled_using_tree())
@@ -397,20 +431,18 @@ static gboolean rebuild_end(GtkMenuItem * root, timestamp_t * ts,
     reset_mnemonics();
     gtk_widget_show_all(GTK_WIDGET(menu_item));
     dirty_gtk_menu_release(menu);
-    tree_count = 0;
     return *ts != 0;
 }
 
 gboolean rebuild_directories(GtkMenuItem * root)
 {
     static GtkMenuItem *menu_item;
-    static DECLARE_TIMESTAMP(timestamp);
     DirtyGtkMenu *menu;
     GNode *tree, *child;
     gchar *prefix, *old_name;
     tree_item *item;
 
-    REBUILD_START(_("Directories"), rebuild_directories);
+    REBUILD_START(_("Directories"), rebuild_directories, directories_timestamp);
     add_current_dir_entry(menu);
 
     /* Build the menu. */
@@ -433,24 +465,23 @@ gboolean rebuild_directories(GtkMenuItem * root)
     } else
         make_menu_from_tree_rec(tree, menu);
 
-    return rebuild_end(root, &timestamp, menu_item, menu);
+    return rebuild_end(root, &directories_timestamp, menu_item, menu);
 }
 
 gboolean rebuild_images(GtkMenuItem * root)
 {
     static GtkMenuItem *menu_item;
-    static DECLARE_TIMESTAMP(timestamp);
     DirtyGtkMenu *menu;
     GNode *tree;
 
-    REBUILD_START(_("Images"), rebuild_images);
+    REBUILD_START(_("Images"), rebuild_images, images_timestamp);
 
     /* Build the menu. */
     add_file_item(NULL, NULL);
     g_node_traverse(tree, G_PRE_ORDER, G_TRAVERSE_LEAFS, -1,
                     (GNodeTraverseFunc) add_file_item, menu);
 
-    return rebuild_end(root, &timestamp, menu_item, menu);
+    return rebuild_end(root, &images_timestamp, menu_item, menu);
 }
 
 /* Rebuild both menus. */
@@ -465,7 +496,19 @@ void set_stop_rebuilding_menu(GtkMenuItem * item)
     gtk_widget_set_sensitive(GTK_WIDGET(item), FALSE);
 }
 
-gboolean is_rebuilding_a_menu(void)
+void obsolete_menus(void)
 {
-    return tree_count != 0;
+    reset_timestamp(&directories_timestamp);
+    reset_timestamp(&images_timestamp);
+}
+
+void cond_rebuild_menus(void)
+{
+    if (directories_timestamp)
+        rebuild_directories(NULL);
+
+    if (images_timestamp)
+        rebuild_images(NULL);
+
+    cond_rebuild_tree_browser();
 }
