@@ -2,7 +2,7 @@
 #define cursor_move_height 20
 #define cursor_move_x_hot 10
 #define cursor_move_y_hot 10
-static unsigned char cursor_move_bits[] = {
+static char cursor_move_bits[] = {
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x04, 0x00, 0x00, 0x0e, 0x00, 0x00, 0x1f, 0x00, 0x00, 0x04, 0x00,
     0x40, 0x44, 0x00, 0x60, 0xc4, 0x00, 0xf0, 0xff, 0x01, 0x60, 0xc4, 0x00,
@@ -11,7 +11,7 @@ static unsigned char cursor_move_bits[] = {
 };
 #define cursor_move_mask_width 20
 #define cursor_move_mask_height 20
-static unsigned char cursor_move_mask_bits[] = {
+static char cursor_move_mask_bits[] = {
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00,
     0x00, 0x0e, 0x00, 0x00, 0x1f, 0x00, 0x80, 0x3f, 0x00, 0x40, 0x5f, 0x00,
     0xe0, 0xee, 0x00, 0xf0, 0xff, 0x01, 0xf8, 0xff, 0x03, 0xf0, 0xff, 0x01,
@@ -22,7 +22,7 @@ static unsigned char cursor_move_mask_bits[] = {
 #define cursor_rect_zoom_height 20
 #define cursor_rect_zoom_x_hot 9
 #define cursor_rect_zoom_y_hot 9
-static unsigned char cursor_rect_zoom_bits[] = {
+static char cursor_rect_zoom_bits[] = {
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x80, 0x07, 0x00, 0x40, 0x08, 0x00, 0x20, 0x10, 0x00, 0x10, 0x20, 0x00,
     0x10, 0x20, 0x00, 0x10, 0x20, 0x00, 0x10, 0x20, 0x00, 0x20, 0x10, 0x00,
@@ -31,7 +31,7 @@ static unsigned char cursor_rect_zoom_bits[] = {
 };
 #define cursor_rect_zoom_mask_width 20
 #define cursor_rect_zoom_mask_height 20
-static unsigned char cursor_rect_zoom_mask_bits[] = {
+static char cursor_rect_zoom_mask_bits[] = {
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x07, 0x00,
     0xc0, 0x0f, 0x00, 0xe0, 0x1f, 0x00, 0x70, 0x38, 0x00, 0x38, 0x70, 0x00,
     0x38, 0x70, 0x00, 0x38, 0x70, 0x00, 0x38, 0x70, 0x00, 0x70, 0x38, 0x00,
@@ -42,7 +42,7 @@ static unsigned char cursor_rect_zoom_mask_bits[] = {
 #define cursor_rotate_height 20
 #define cursor_rotate_x_hot 12
 #define cursor_rotate_y_hot 11
-static unsigned char cursor_rotate_bits[] = {
+static char cursor_rotate_bits[] = {
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x00,
     0x40, 0xdc, 0x01, 0xc0, 0x06, 0x03, 0x40, 0x03, 0x06, 0x40, 0x02, 0x04,
     0x40, 0x04, 0x00, 0xc0, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -51,7 +51,7 @@ static unsigned char cursor_rotate_bits[] = {
 };
 #define cursor_rotate_mask_width 20
 #define cursor_rotate_mask_height 20
-static unsigned char cursor_rotate_mask_bits[] = {
+static char cursor_rotate_mask_bits[] = {
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x00, 0x60, 0xfc, 0x01,
     0xe0, 0xfe, 0x03, 0xe0, 0xdf, 0x07, 0xe0, 0x07, 0x0f, 0xe0, 0x07, 0x0e,
     0xe0, 0x0f, 0x04, 0xe0, 0x1f, 0x00, 0xe0, 0x1f, 0x00, 0x00, 0xf0, 0x0f,
@@ -62,7 +62,7 @@ static unsigned char cursor_rotate_mask_bits[] = {
 #define cursor_zoom_height 20
 #define cursor_zoom_x_hot 9
 #define cursor_zoom_y_hot 9
-static unsigned char cursor_zoom_bits[] = {
+static char cursor_zoom_bits[] = {
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x80, 0x07, 0x00, 0x40, 0x08, 0x00, 0x20, 0x10, 0x00, 0x10, 0x20, 0x00,
     0x10, 0x20, 0x00, 0x10, 0x20, 0x00, 0x10, 0x20, 0x00, 0x20, 0x10, 0x00,
@@ -71,7 +71,7 @@ static unsigned char cursor_zoom_bits[] = {
 };
 #define cursor_zoom_mask_width 20
 #define cursor_zoom_mask_height 20
-static unsigned char cursor_zoom_mask_bits[] = {
+static char cursor_zoom_mask_bits[] = {
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x07, 0x00,
     0xc0, 0x0f, 0x00, 0xe0, 0x1f, 0x00, 0x70, 0x38, 0x00, 0x38, 0x70, 0x00,
     0x38, 0x70, 0x00, 0x38, 0x70, 0x00, 0x38, 0x70, 0x00, 0x70, 0x38, 0x00,
@@ -82,7 +82,7 @@ static unsigned char cursor_zoom_mask_bits[] = {
 #define cursor_zoom_rotate_height 20
 #define cursor_zoom_rotate_x_hot 12
 #define cursor_zoom_rotate_y_hot 11
-static unsigned char cursor_zoom_rotate_bits[] = {
+static char cursor_zoom_rotate_bits[] = {
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x00,
     0xc0, 0xdf, 0x01, 0xc0, 0x0e, 0x03, 0x60, 0x13, 0x06, 0x50, 0x22, 0x04,
     0x50, 0x24, 0x00, 0xd0, 0x2f, 0x00, 0x10, 0x20, 0x00, 0x20, 0x10, 0x00,
@@ -91,7 +91,7 @@ static unsigned char cursor_zoom_rotate_bits[] = {
 };
 #define cursor_zoom_rotate_mask_width 20
 #define cursor_zoom_rotate_mask_height 20
-static unsigned char cursor_zoom_rotate_mask_bits[] = {
+static char cursor_zoom_rotate_mask_bits[] = {
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x00, 0xe0, 0xff, 0x01,
     0xe0, 0xff, 0x03, 0xe0, 0xdf, 0x07, 0xf0, 0x3f, 0x0f, 0xf8, 0x77, 0x0e,
     0xf8, 0x7f, 0x04, 0xf8, 0x7f, 0x00, 0xf8, 0x7f, 0x00, 0x70, 0xf8, 0x0f,
