@@ -6,7 +6,7 @@
 
 gint serialize_collection_nogui(const gchar * filename);
 gboolean serialize_collection_gui(void);
-gint load_dot_gliv_from_file(FILE * file);
+gint load_dot_gliv_from_file(const gchar * filename, FILE * file);
 gint load_dot_gliv(const gchar * filename);
 gboolean deserialize_collection(void);
 
