@@ -4,7 +4,7 @@
 #include "gliv.h"
 
 typedef struct {
-    GtkMenuShell *gtk_menu_shell;
+    GtkMenu *gtk_menu;
     GList *last_child;
     GtkTearoffMenuItem *tearoff;
 } DirtyGtkMenu;
