@@ -2,12 +2,12 @@
 #define FILES_LIST_H
 
 #include "gliv.h"
+#include "gliv-image.h"
 #include "timestamp.h"
 
 gint get_list_length(void);
 GList *get_list_head(void);
 GList *get_list_end(void);
-gint add_file_to_list(const gchar * filename);
 void remove_from_list(GList * node);
 gboolean reorder_files(gboolean shuffle);
 gchar **get_sorted_files_array(void);
@@ -18,8 +18,10 @@ gint init_list(gchar ** names, gint nb, gboolean sort, gboolean shuffle,
 gint insert_after_current(gchar ** names, gint nb, gboolean just_file,
                           gboolean add_all);
 gboolean confirm_remove_current(void);
-void delete_selected_image(void);
+gboolean delete_selected_image(void);
 timestamp_t get_list_timestamp(void);
 const gchar *get_nth_filename(gint n);
+gint get_image_number(GlivImage * im);
+GList *find_node_by_name(const gchar * name);
 
 #endif
