@@ -30,15 +30,24 @@
 #define OPTION_BACKGROUND                (&new_options->bg_col)
 #define OPTION_ALPHA1                    (&new_options->alpha1)
 #define OPTION_ALPHA2                    (&new_options->alpha2)
-#define OPTION_NO_CENTER                 (&new_options->no_center)
 #define OPTION_NOTICE_TIME               (&new_options->notice_time)
 #define OPTION_CONFIRM_QUIT              (&new_options->confirm_quit)
-#define OPTION_TRANS_STEPS               (&new_options->trans_steps)
-#define OPTION_TRANS_SLEEP               (&new_options->trans_sleep)
+#define OPTION_TRANS_DURATION            (&new_options->trans_time)
 #define OPTION_SAVE_QUIT                 (&new_options->save_quit)
 #define OPTION_TRANSITIONS               (&new_options->transitions)
 #define OPTION_KEEP_TRANSFO              (&new_options->keep_transfo)
 #define OPTION_OPENGL_ERRORS             (&new_options->opengl_errors)
+#define OPTION_FILTERING                 (&new_options->filtering)
+
+/* Must be kept in sync with 'enum transfo' */
+enum image_position {
+    POSITION_CENTER = 0,
+    POSITION_TOP_LEFT,
+    POSITION_TOP_RIGHT,
+    POSITION_BOTTOM_LEFT,
+    POSITION_BOTTOM_RIGHT,
+    POSITION_KEEP
+};
 
 /* Options controlled by the user. */
 typedef struct {
@@ -60,13 +69,13 @@ typedef struct {
     gboolean start_show;
     gboolean thumbnails;
     gboolean resize_win;
-    gboolean no_center;
     gboolean confirm_quit;
     gboolean save_quit;
     gboolean transitions;
     gboolean recursive;
     gboolean keep_transfo;
     gboolean opengl_errors;
+    gboolean filtering;
     gint thumb_width;
     gint thumb_height;
     gint delay;
@@ -74,8 +83,9 @@ typedef struct {
     gint history_size;
     gint fps;
     gint notice_time;
-    gint trans_steps;
-    gint trans_sleep;
+    gint trans_time;
+    enum image_position initial_pos;
+    gchar *initial_geometry;
     GdkColor bg_col;
     GdkColor alpha1;
     GdkColor alpha2;
