@@ -225,9 +225,7 @@ static void process_data(gchar * data, gint length, gchar * client_dir)
 
     tab = rebuild_args_array(data, length, &count, client_dir);
     nb_inserted = insert_after_current(tab, count, TRUE, FALSE);
-
-    if (nb_inserted > 0)
-        open_next_image(nb_inserted == 1);
+    new_images(nb_inserted);
 
     for (i = 0; i < count; i++)
         g_free(tab[i]);
