@@ -48,6 +48,7 @@
 #include "ipc.h"
 #include "actions.h"
 #include "tree.h"
+#include "tree_browser.h"
 
 extern options_struct *options;
 extern GtkMenuBar *menu_bar;
@@ -99,6 +100,9 @@ static GtkMenuShell *add_menu(const gchar * name)
     return menu;
 }
 
+/* Should not collide with a keysym */
+#define CTRL (1 << 31)
+
 static GtkMenuItem *add_menu_item(GtkMenuShell * menu,
                                   const gchar * menu_name, const gchar * name,
                                   GCallback func, guint key, gpointer data)
@@ -117,7 +121,13 @@ static GtkMenuItem *add_menu_item(GtkMenuShell * menu,
 
     g_signal_connect_swapped(menu_item, "activate", func, data);
 
-    mod = (key >= GDK_A && key <= GDK_Z) ? GDK_SHIFT_MASK : 0;
+    if (key >= GDK_A && key <= GDK_Z)
+        mod = GDK_SHIFT_MASK;
+    else if (key & CTRL) {
+        key &= ~CTRL;
+        mod = GDK_CONTROL_MASK;
+    } else
+        mod = 0;
 
     accel_path = g_strconcat("<GLiv>/", menu_name, "/", name, NULL);
     gtk_accel_map_add_entry(accel_path, key, mod);
@@ -163,180 +173,210 @@ static gboolean menu_maximize(gint what)
     return FALSE;
 }
 
-enum {
-    TRANSFORMATION_MOVE_LEFT,
-    TRANSFORMATION_MOVE_RIGHT,
-    TRANSFORMATION_MOVE_UP,
-    TRANSFORMATION_MOVE_DOWN,
+enum transfo {
+    /* Must be kept in sync with 'enum image_position' */
+    TRANSFO_CENTER = 0,
+    TRANSFO_TOP_LEFT,
+    TRANSFO_TOP_RIGHT,
+    TRANSFO_BOTTOM_LEFT,
+    TRANSFO_BOTTOM_RIGHT,
     /* --- */
-    TRANSFORMATION_ROTATE_BIG_PLUS,
-    TRANSFORMATION_ROTATE_BIG_MINUS,
-    TRANSFORMATION_ROTATE_SMALL_PLUS,
-    TRANSFORMATION_ROTATE_SMALL_MINUS,
+    TRANSFO_MOVE_LEFT,
+    TRANSFO_MOVE_RIGHT,
+    TRANSFO_MOVE_UP,
+    TRANSFO_MOVE_DOWN,
+    /* --- */
+    TRANSFO_ROTATE_BIG_PLUS,
+    TRANSFO_ROTATE_BIG_MINUS,
+    TRANSFO_ROTATE_SMALL_PLUS,
+    TRANSFO_ROTATE_SMALL_MINUS,
+    /* --- */
+    TRANSFO_ZOOM_IN,
+    TRANSFO_ZOOM_OUT,
+    TRANSFO_FIT_WIDTH,
+    TRANSFO_FIT_HEIGHT,
+    TRANSFO_FLIP_H,
+    TRANSFO_FLIP_V,
+    TRANSFO_RESET,
     /* --- */
-    TRANSFORMATION_ZOOM_IN,
-    TRANSFORMATION_ZOOM_OUT,
-    TRANSFORMATION_FLIP_H,
-    TRANSFORMATION_FLIP_V,
-    TRANSFORMATION_RESET
 };
 
-static guchar menu_move(guchar action)
+static gboolean menu_transfo(enum transfo transfo)
 {
-    switch (action) {
-    case TRANSFORMATION_MOVE_LEFT:
+    switch (transfo) {
+    case TRANSFO_CENTER:
+    case TRANSFO_TOP_LEFT:
+    case TRANSFO_TOP_RIGHT:
+    case TRANSFO_BOTTOM_LEFT:
+    case TRANSFO_BOTTOM_RIGHT:
+        {
+            enum image_position old_pos = options->initial_pos;
+            options->initial_pos = transfo;
+            matrix_set_initial_position();
+            options->initial_pos = old_pos;
+            break;
+        }
+
+        /* --- */
+
+    case TRANSFO_MOVE_LEFT:
         matrix_move(MOVE_OFFSET, 0.0);
         break;
 
-    case TRANSFORMATION_MOVE_RIGHT:
+    case TRANSFO_MOVE_RIGHT:
         matrix_move(-MOVE_OFFSET, 0.0);
         break;
 
-    case TRANSFORMATION_MOVE_UP:
+    case TRANSFO_MOVE_UP:
         matrix_move(0.0, MOVE_OFFSET);
         break;
 
-        /* case TRANSFORMATION_MOVE_DOWN: */
-    default:
+    case TRANSFO_MOVE_DOWN:
         matrix_move(0.0, -MOVE_OFFSET);
-    }
+        break;
 
-    return REFRESH_IMAGE | APPEND_HISTORY;
-}
+        /* --- */
 
-static guchar menu_rotate(guchar action)
-{
-    switch (action) {
-    case TRANSFORMATION_ROTATE_BIG_PLUS:
+    case TRANSFO_ROTATE_BIG_PLUS:
         matrix_rotate(BIG_ROTATION);
         break;
 
-    case TRANSFORMATION_ROTATE_BIG_MINUS:
+    case TRANSFO_ROTATE_BIG_MINUS:
         matrix_rotate(-BIG_ROTATION);
         break;
 
-    case TRANSFORMATION_ROTATE_SMALL_PLUS:
+    case TRANSFO_ROTATE_SMALL_PLUS:
         matrix_rotate(SMALL_ROTATION);
         break;
 
-        /* case TRANSFORMATION_ROTATE_SMALL_MINUS: */
-    default:
+    case TRANSFO_ROTATE_SMALL_MINUS:
         matrix_rotate(-SMALL_ROTATION);
-    }
-
-    return REFRESH_IMAGE | REFRESH_STATUS | APPEND_HISTORY;
-}
-
-gboolean menu_set_top_left(void)
-{
-    matrix_set_top_left();
-    refresh(REFRESH_IMAGE | APPEND_HISTORY);
-    return FALSE;
-}
-
-static void menu_zoom(guchar action)
-{
-    gboolean zoom_pointer;
-
-    zoom_pointer = options->zoom_pointer;
-    options->zoom_pointer = FALSE;
-
-    if (action == TRANSFORMATION_ZOOM_IN)
-        zoom_in(ZOOM_FACTOR);
-    else
-        zoom_in(1.0 / ZOOM_FACTOR);
-
-    options->zoom_pointer = zoom_pointer;
-}
-
-static gboolean menu_transformation(guchar action)
-{
-    gint what = 0;
-
-    switch (action) {
-    case TRANSFORMATION_MOVE_LEFT:
-    case TRANSFORMATION_MOVE_RIGHT:
-    case TRANSFORMATION_MOVE_UP:
-    case TRANSFORMATION_MOVE_DOWN:
-        what = menu_move(action);
         break;
 
-    case TRANSFORMATION_ROTATE_BIG_PLUS:
-    case TRANSFORMATION_ROTATE_BIG_MINUS:
-    case TRANSFORMATION_ROTATE_SMALL_PLUS:
-    case TRANSFORMATION_ROTATE_SMALL_MINUS:
-        what = menu_rotate(action);
+        /* --- */
+
+    case TRANSFO_ZOOM_IN:
+    case TRANSFO_ZOOM_OUT:
+        {
+            gboolean zoom_pointer = options->zoom_pointer;
+            options->zoom_pointer = FALSE;
+
+            if (transfo == TRANSFO_ZOOM_IN)
+                zoom_in(ZOOM_FACTOR);
+            else
+                zoom_in(1.0 / ZOOM_FACTOR);
+
+            options->zoom_pointer = zoom_pointer;
+            break;
+        }
+
+    case TRANSFO_FIT_WIDTH:
+        matrix_fit(TRUE, FALSE);
         break;
 
-    case TRANSFORMATION_ZOOM_IN:
-    case TRANSFORMATION_ZOOM_OUT:
-        menu_zoom(action);
+    case TRANSFO_FIT_HEIGHT:
+        matrix_fit(FALSE, TRUE);
         break;
 
-    case TRANSFORMATION_FLIP_H:
+    case TRANSFO_FLIP_H:
         matrix_flip_h();
-        what = REFRESH_IMAGE | REFRESH_STATUS | APPEND_HISTORY;
         break;
 
-    case TRANSFORMATION_FLIP_V:
+    case TRANSFO_FLIP_V:
         matrix_flip_v();
-        what = REFRESH_IMAGE | REFRESH_STATUS | APPEND_HISTORY;
         break;
 
-        /* case TRANSFORMATION_RESET: */
-    default:
+    case TRANSFO_RESET:
         matrix_reset();
-        what = REFRESH_IMAGE | REFRESH_STATUS | APPEND_HISTORY;
+        break;
+
+    default:
+        g_printerr("Unknown transformation: %d\n", transfo);
+        return FALSE;
     }
 
-    refresh(what);
+    refresh(REFRESH_IMAGE | REFRESH_STATUS | APPEND_HISTORY);
     return FALSE;
 }
 
-#define SEPARATOR gtk_menu_shell_append(submenu, gtk_separator_menu_item_new())
+#define SEPARATOR gtk_menu_shell_append(menu, gtk_separator_menu_item_new())
 
-#define ADD_MENU_ITEM(str, signal_func, key, val)                        \
-    add_menu_item(submenu, menu_name, str, G_CALLBACK(signal_func), key, \
+#define ADD_MENU_ITEM(str, signal_func, key, val)                     \
+    add_menu_item(menu, menu_name, str, G_CALLBACK(signal_func), key, \
                   GINT_TO_POINTER(val))
 
-#define ADD_MENU(str)                                             \
-    do {                                                          \
-        submenu = add_menu(str);                                  \
-        menu_name = str;                                          \
-        gtk_menu_set_accel_group(GTK_MENU(submenu), accel_group); \
+#define ADD_MENU(str)                                          \
+    do {                                                       \
+        menu = add_menu(str);                                  \
+        menu_name = str;                                       \
+        gtk_menu_set_accel_group(GTK_MENU(menu), accel_group); \
     } while (0)
 
 /* Used to add an images menu. */
 static GtkMenuItem *add_special_menu(const gchar * name, GCallback rebuild_menu)
 {
     GList *children;
-    GtkMenuShell *submenu;
+    GtkMenuShell *menu;
     GtkMenuItem *item;
     gchar *menu_name = NULL;
 
-    submenu = add_menu(name);
+    menu = add_menu(name);
 
-    children = gtk_container_get_children(GTK_CONTAINER(submenu));
-    item = GTK_MENU_ITEM(gtk_menu_get_attach_widget(GTK_MENU(submenu)));
+    children = gtk_container_get_children(GTK_CONTAINER(menu));
+    item = GTK_MENU_ITEM(gtk_menu_get_attach_widget(GTK_MENU(menu)));
     g_list_free(children);
 
     ADD_MENU_ITEM(_("Rebuild this menu"), rebuild_menu, 0, 0);
+    ADD_MENU_ITEM(_("Open thumbnails browser..."), show_tree_browser, 0, 0);
 
     return item;
 }
 
 #define ADD_SPECIAL_MENU(str, cb) cb(add_special_menu(str, G_CALLBACK(cb)))
 
+static GtkMenuShell *add_submenu(GtkMenuShell * menu, const gchar * name)
+{
+    GtkMenuItem *item;
+    GtkMenu *submenu;
+
+    item = add_menu_item(menu, NULL, name, NULL, 0, NULL);
+    submenu = GTK_MENU(gtk_menu_new());
+
+    gtk_menu_item_set_submenu(item, GTK_WIDGET(submenu));
+    gtk_menu_shell_append(GTK_MENU_SHELL(submenu), gtk_tearoff_menu_item_new());
+    push_mnemonics();
+
+    return GTK_MENU_SHELL(submenu);
+}
+
+
+#define ADD_SUBMENU(name)                                         \
+    do {                                                          \
+        submenu = add_submenu(menu, name);                        \
+        gtk_menu_set_accel_group(GTK_MENU(submenu), accel_group); \
+        submenu_name = g_strconcat(menu_name, "/", name, NULL);   \
+    } while (0)
+
+#define END_SUBMENU()         \
+    do {                      \
+        pop_mnemonics();      \
+        g_free(submenu_name); \
+    } while (0)
+
+#define ADD_SUBMENU_ITEM(name, action, key)                                   \
+    add_menu_item(submenu, submenu_name, name, G_CALLBACK(menu_transfo), key, \
+                  GINT_TO_POINTER(action))
+
 /* GtkAction are not supported in gtk+-2 < 2.3 */
 extern const guint gtk_minor_version;
 
 
 GtkAccelGroup *create_menus(void)
 {
-    GtkMenuShell *submenu;
+    GtkMenuShell *menu, *submenu;
     GtkAccelGroup *accel_group;
     GtkMenuItem *item, *current_image_item, *every_image_item;
-    gchar *menu_name;
+    gchar *menu_name, *submenu_name;
 
     menu_bar = GTK_MENU_BAR(gtk_menu_bar_new());
 
@@ -364,22 +404,10 @@ GtkAccelGroup *create_menus(void)
     /******************/
     /* Commands menu. */
     ADD_MENU(N_("Commands"));
-    ADD_MENU_ITEM(N_("Move top left"), menu_set_top_left, GDK_Home, 0);
-    ADD_MENU_ITEM(N_("Maximize small image"), menu_maximize, GDK_M, MAXIMIZE);
-    ADD_MENU_ITEM(N_("Scale down large image"), menu_maximize, GDK_l,
-                  SCALE_DOWN);
-    ADD_MENU_ITEM(N_("Image fit window"), menu_maximize, GDK_m,
-                  MAXIMIZE | SCALE_DOWN);
-    SEPARATOR;                  /* ------------------------------------ */
-    ADD_MENU_ITEM(N_("Horizontal flip"), menu_transformation, GDK_z,
-                  TRANSFORMATION_FLIP_H);
-
-    ADD_MENU_ITEM(N_("Vertical flip"), menu_transformation, GDK_e,
-                  TRANSFORMATION_FLIP_V);
-    SEPARATOR;                  /* ------------------------------------ */
     ADD_MENU_ITEM(N_("Load previous image"), load_direction, GDK_p, -1);
     ADD_MENU_ITEM(N_("Load next image"), load_direction, GDK_n, 1);
     ADD_MENU_ITEM(N_("Load the Nth image..."), menu_image_nr, GDK_g, 0);
+    ADD_MENU_ITEM(N_("Reload the current image"), reload_current_image, 0, 0);
     SEPARATOR;                  /* ------------------------------------ */
     ADD_MENU_ITEM(N_("Sort images list"), reorder_files, 0, FALSE);
     ADD_MENU_ITEM(N_("Shuffle images list"), reorder_files, 0, TRUE);
@@ -389,14 +417,16 @@ GtkAccelGroup *create_menus(void)
                          cancel_using_tree, 0, 0);
     set_stop_rebuilding_menu(item);
     SEPARATOR;                  /* ------------------------------------ */
+    ADD_MENU_ITEM(N_("Toggle floating windows"), toggle_floating_windows,
+                  GDK_w, 0);
     item = ADD_MENU_ITEM(N_("Start the slide show"), toggle_slide_show,
                          GDK_Pause, 0);
     set_slide_show_menu(item);
 
-    ADD_MENU_ITEM(N_("Remove the current file"), confirm_remove_current, GDK_x,
-                  0);
+    ADD_MENU_ITEM(N_("Remove the current file"), confirm_remove_current,
+                  GDK_Delete, 0);
     SEPARATOR;                  /* ------------------------------------ */
-    ADD_MENU_ITEM(N_("Edit actions"), edit_actions, 0, 0);
+    ADD_MENU_ITEM(N_("Edit actions..."), edit_actions, 0, 0);
     current_image_item = ADD_MENU_ITEM(_("Action on the current image"),
                                        NULL, 0, 0);
     every_image_item = ADD_MENU_ITEM(_("Action on every image"), NULL, 0, 0);
@@ -408,44 +438,54 @@ GtkAccelGroup *create_menus(void)
     ADD_MENU_ITEM(N_("Redo"), redo, GDK_y, 0);
     ADD_MENU_ITEM(N_("Clear history"), clear_history, GDK_c, 0);
     SEPARATOR;                  /* ------------------------------------ */
-    ADD_MENU_ITEM(N_("Move left"), menu_transformation, 0,
-                  TRANSFORMATION_MOVE_LEFT);
+    ADD_SUBMENU(N_("Move"));
+    ADD_SUBMENU_ITEM(N_("Move left"), TRANSFO_MOVE_LEFT, GDK_Left);
+    ADD_SUBMENU_ITEM(N_("Move right"), TRANSFO_MOVE_RIGHT, GDK_Right);
+    ADD_SUBMENU_ITEM(N_("Move up"), TRANSFO_MOVE_UP, GDK_Up);
+    ADD_SUBMENU_ITEM(N_("Move down"), TRANSFO_MOVE_DOWN, GDK_Down);
+    END_SUBMENU();
 
-    ADD_MENU_ITEM(N_("Move right"), menu_transformation, 0,
-                  TRANSFORMATION_MOVE_RIGHT);
+    ADD_SUBMENU(N_("Set position"));
+    ADD_SUBMENU_ITEM(N_("Center"), TRANSFO_CENTER, 0);
+    ADD_SUBMENU_ITEM(N_("Top left"), TRANSFO_TOP_LEFT, GDK_Home);
+    ADD_SUBMENU_ITEM(N_("Top right"), TRANSFO_TOP_RIGHT, 0);
+    ADD_SUBMENU_ITEM(N_("Bottom left"), TRANSFO_BOTTOM_LEFT, 0);
+    ADD_SUBMENU_ITEM(N_("Bottom right"), TRANSFO_BOTTOM_RIGHT, 0);
+    END_SUBMENU();
 
-    ADD_MENU_ITEM(N_("Move up"), menu_transformation, 0,
-                  TRANSFORMATION_MOVE_UP);
-
-    ADD_MENU_ITEM(N_("Move down"), menu_transformation, 0,
-                  TRANSFORMATION_MOVE_DOWN);
+    ADD_SUBMENU(N_("Rotation"));
+    ADD_SUBMENU_ITEM(N_("Rotate +90 deg"), TRANSFO_ROTATE_BIG_PLUS,
+                     CTRL | GDK_Up);
+    ADD_SUBMENU_ITEM(N_("Rotate -90 deg"), TRANSFO_ROTATE_BIG_MINUS,
+                     CTRL | GDK_Down);
+    ADD_SUBMENU_ITEM(N_("Rotate +0.1 deg"), TRANSFO_ROTATE_SMALL_PLUS,
+                     CTRL | GDK_Left);
+    ADD_SUBMENU_ITEM(N_("Rotate -0.1 deg"), TRANSFO_ROTATE_SMALL_MINUS,
+                     CTRL | GDK_Right);
+    END_SUBMENU();
 
     SEPARATOR;                  /* ------------------------------------ */
 
-    ADD_MENU_ITEM(N_("Rotate +90 deg"), menu_transformation, 0,
-                  TRANSFORMATION_ROTATE_BIG_PLUS);
+    ADD_MENU_ITEM(N_("Horizontal flip"), menu_transfo, GDK_z, TRANSFO_FLIP_H);
 
-    ADD_MENU_ITEM(N_("Rotate -90 deg"), menu_transformation, 0,
-                  TRANSFORMATION_ROTATE_BIG_MINUS);
-
-    ADD_MENU_ITEM(N_("Rotate +0.1 deg"), menu_transformation, 0,
-                  TRANSFORMATION_ROTATE_SMALL_PLUS);
-
-    ADD_MENU_ITEM(N_("Rotate -0.1 deg"), menu_transformation, 0,
-                  TRANSFORMATION_ROTATE_SMALL_MINUS);
+    ADD_MENU_ITEM(N_("Vertical flip"), menu_transfo, GDK_e, TRANSFO_FLIP_V);
 
     SEPARATOR;                  /* ------------------------------------ */
 
-    ADD_MENU_ITEM(N_("Zoom in"), menu_transformation, 0,
-                  TRANSFORMATION_ZOOM_IN);
+    ADD_MENU_ITEM(N_("Maximize small image"), menu_maximize, GDK_M, MAXIMIZE);
+    ADD_MENU_ITEM(N_("Scale down large image"), menu_maximize, GDK_l,
+                  SCALE_DOWN);
+    ADD_MENU_ITEM(N_("Image fit window"), menu_maximize, GDK_m,
+                  MAXIMIZE | SCALE_DOWN);
 
-    ADD_MENU_ITEM(N_("Zoom out"), menu_transformation, 0,
-                  TRANSFORMATION_ZOOM_OUT);
+    ADD_MENU_ITEM(N_("Zoom in"), menu_transfo, GDK_plus, TRANSFO_ZOOM_IN);
+    ADD_MENU_ITEM(N_("Zoom out"), menu_transfo, GDK_minus, TRANSFO_ZOOM_OUT);
+    ADD_MENU_ITEM(N_("Fit width"), menu_transfo, 0, TRANSFO_FIT_WIDTH);
+    ADD_MENU_ITEM(N_("Fit height"), menu_transfo, 0, TRANSFO_FIT_HEIGHT);
 
     SEPARATOR;                  /* ------------------------------------ */
 
-    ADD_MENU_ITEM(N_("Reset"), menu_transformation, GDK_r,
-                  TRANSFORMATION_RESET);
+    ADD_MENU_ITEM(N_("Reset"), menu_transfo, GDK_r, TRANSFO_RESET);
 
     /*****************/
     /* Options menu. */
