@@ -40,8 +40,9 @@
 #include "menus.h"
 #include "callbacks.h"
 #include "transition.h"
-#include "formats.h"
 #include "windows.h"
+#include "tree_browser.h"
+#include "images_menus.h"
 
 extern rt_struct *rt;
 extern options_struct *options;
@@ -156,21 +157,26 @@ static void render_next_image(GlivImage * im)
 {
     gboolean first_image = current_image == NULL;
     gboolean make_transition = options->transitions &&
-        options->trans_steps > 0 && current_image != NULL && im != NULL;
+        options->trans_time > 0 && current_image != NULL && im != NULL;
 
-    if (make_transition) {
-
-        if (options->fullscreen == FALSE)
-            goto_window(im, FALSE);
-
-        /* Be sure to update the display now. A transition will take place ... */
-        gtk_widget_queue_draw(GTK_WIDGET(get_current_window()));
-        gdk_window_process_all_updates();
-        process_events();
-
-        transition(im);
+    if (options->fullscreen == FALSE) {
+        gint i;
+        goto_window(im, FALSE);
+        /*
+         * This seems to work to make sure the window is actually resized and
+         * the corresponding events have been handled.
+         */
+        for (i = 0; i < 2; i++) {
+            gtk_widget_queue_draw(GTK_WIDGET(get_current_window()));
+            gdk_window_process_all_updates();
+            process_events();
+            gdk_flush();
+        }
     }
 
+    if (make_transition)
+        transition(im);
+
     if (current_image != NULL) {
         prioritize_textures(current_image, FALSE);
         g_object_unref(current_image);
@@ -180,13 +186,6 @@ static void render_next_image(GlivImage * im)
     g_object_ref(im);
     render();
 
-    if (make_transition == FALSE)
-        if (options->fullscreen == FALSE)
-            goto_window(im, FALSE);
-
-    if (options->no_center)
-        menu_set_top_left();
-
     if (im->node->next == NULL)
         image_notice(TRUE);
 
@@ -195,6 +194,8 @@ static void render_next_image(GlivImage * im)
 
     if (slide_show_started())
         slide_show_advance();
+
+    highlight_current_image();
 }
 
 /*** Next/previous image ***/
@@ -220,11 +221,12 @@ static void show_errors(GSList * errors)
 
     /* Create the widgets */
 
-    dialog = GTK_DIALOG(gtk_dialog_new_with_buttons(_("Loading errors"),
-                                                    get_current_window(),
-                                                    GTK_DIALOG_DESTROY_WITH_PARENT,
-                                                    GTK_STOCK_OK,
-                                                    GTK_RESPONSE_NONE, NULL));
+    dialog =
+        GTK_DIALOG(gtk_dialog_new_with_buttons(_("Loading errors"),
+                                               get_current_window(),
+                                               GTK_DIALOG_DESTROY_WITH_PARENT,
+                                               GTK_STOCK_OK,
+                                               GTK_RESPONSE_NONE, NULL));
 
     text = GTK_TEXT_VIEW(gtk_text_view_new());
     gtk_text_view_set_wrap_mode(text, GTK_WRAP_WORD_CHAR);
@@ -349,7 +351,9 @@ static GlivImage *load_in_direction(gint dir)
     if (im != NULL) {
         im->node = node;
         im->number = id;
-    }
+    } else if (get_list_head() == NULL && current_image->node)
+        /* No more images, not even the current one */
+        current_image->node = NULL;
 
     if (before_load != get_list_timestamp())
         update_current_image_status(TRUE);
@@ -369,16 +373,6 @@ static void destroy_unused(GlivImage ** backup)
     g_object_ref(*backup);
 }
 
-static void load_next(void)
-{
-    load_direction(1);
-}
-
-static void load_prev(void)
-{
-    load_direction(-1);
-}
-
 static void stop_slide_show(void);
 
 /* Return FALSE if there are no more images. */
@@ -390,57 +384,59 @@ void load_direction(gint dir)
 
     /* We are not reentrant */
     static gboolean in_load_direction = FALSE;
+    static gint next_direction = 0;
+
+    if (in_load_direction) {
+        /* We delay the reentrance */
+        next_direction = dir;
+        return;
+    }
 
     if (currently_loading() || current_image == NULL)
         return;
 
-    if (dir == -1) {
-        backup = &next_image;
-        next = &previous_image;
-    } else if (dir == 1) {
-        backup = &previous_image;
-        next = &next_image;
-    } else
-        return;
+    for (;;) {
+        if (dir == -1) {
+            backup = &next_image;
+            next = &previous_image;
+        } else if (dir == 1) {
+            backup = &previous_image;
+            next = &next_image;
+        } else
+            return;
 
-    if (in_load_direction) {
-        /* We delay the reentrance */
-        if (dir > 0)
-            do_later(G_PRIORITY_DEFAULT, load_next);
-        else
-            do_later(G_PRIORITY_DEFAULT, load_prev);
-        return;
-    }
+        in_load_direction = TRUE;
+        next_direction = 0;
 
-    in_load_direction = TRUE;
-
-    /* Be sure there is a next image. */
-    if (*next == NULL) {
-        *next = load_in_direction(dir);
+        /* Be sure there is a next image. */
         if (*next == NULL) {
-            /* There is no next image to load. */
-            image_notice(dir == 1);
-            stop_slide_show();
-            goto out;
+            *next = load_in_direction(dir);
+            if (*next == NULL) {
+                /* There is no next image to load. */
+                image_notice(dir == 1);
+                stop_slide_show();
+                goto out;
+            }
         }
+
+        destroy_unused(backup);
+        im = *next;
+        *next = NULL;
+
+        render_next_image(im);
+        g_object_unref(im);
+
+        if (options->one_image) {
+            g_object_unref(*backup);
+            *backup = NULL;
+        } else
+            *next = load_in_direction(dir);
+
+        slide_show_direction = dir;
+      out:
+        in_load_direction = FALSE;
+        dir = next_direction;
     }
-
-    destroy_unused(backup);
-    im = *next;
-    *next = NULL;
-
-    render_next_image(im);
-    g_object_unref(im);
-
-    if (options->one_image) {
-        g_object_unref(*backup);
-        *backup = NULL;
-    } else
-        *next = load_in_direction(dir);
-
-    slide_show_direction = dir;
-  out:
-    in_load_direction = FALSE;
 }
 
 /*
@@ -449,7 +445,7 @@ void load_direction(gint dir)
  * Maybe the image after the one we will load does
  * not change.
  */
-void open_next_image(gboolean keep_next)
+static void open_next_image(gboolean keep_next)
 {
     GlivImage *next;
     GlivImage *future_image;
@@ -486,31 +482,43 @@ void open_next_image(gboolean keep_next)
 
 /*** First/second image ***/
 
-GlivImage *load_first_image(void)
-{
-    GlivImage *im;
-
-    install_segv_handler();
-
-    if (get_list_head() == NULL)
-        return NULL;
-
-    im = load_in_direction(1);
-    if (im == NULL)
-        DIALOG_MSG(_("No image found"));
-    else
-        render_next_image(im);
-
-    return im;
-}
-
-void load_second_image(void)
+static void load_second_image(void)
 {
     if (next_image == NULL && options->one_image == FALSE)
         next_image = load_in_direction(1);
 
     if (options->start_show)
         start_slide_show();
+
+    if (options->build_menus)
+        do_later(G_PRIORITY_LOW, (GSourceDummyMarshal) rebuild_images_menus);
+}
+
+void load_first_image(void)
+{
+    GlivImage *first_image;
+
+    install_segv_handler();
+
+    if (get_list_head() == NULL)
+        return;
+
+    first_image = load_in_direction(1);
+    if (first_image == NULL)
+        DIALOG_MSG(_("No image found"));
+    else
+        render_next_image(first_image);
+
+    if (first_image != NULL) {
+        if (options->fullscreen == FALSE && options->resize_win == FALSE)
+            goto_window(first_image, TRUE);
+
+        g_object_unref(first_image);
+
+    } else if (options->fullscreen == FALSE)
+        goto_window(NULL, TRUE);
+
+    do_later(G_PRIORITY_LOW, load_second_image);
 }
 
 /*
@@ -518,11 +526,12 @@ void load_second_image(void)
  * Used for example, when an option is changed such as dithering or mipmap.
  */
 
-static void reload_current_image(void)
+void reload_current_image(void)
 {
     GlivImage *new;
 
-    if (currently_loading() || current_image == NULL)
+    if (currently_loading() || current_image == NULL ||
+        current_image->node == NULL)
         return;
 
     new = load_file(current_image->node->data, NULL);
@@ -562,15 +571,16 @@ struct image_stat3 {
     struct image_stat next;
 };
 
-static void stat_image(GlivImage * im, struct image_stat *st)
+static void stat_image(GlivImage * im, struct image_stat *st, gboolean quiet)
 {
     st->present = FALSE;
 
-    if (im != NULL) {
+    if (im != NULL && im->node != NULL) {
         struct stat real_stat;
 
         if (stat(im->node->data, &real_stat) < 0) {
-            perror(im->node->data);
+            if (!quiet)
+                perror(im->node->data);
             return;
         }
 
@@ -580,13 +590,13 @@ static void stat_image(GlivImage * im, struct image_stat *st)
     }
 }
 
-gpointer stat_loaded_files(void)
+gpointer stat_loaded_files(gboolean quiet)
 {
     struct image_stat3 *stat_data = g_new(struct image_stat3, 1);
 
-    stat_image(previous_image, &stat_data->previous);
-    stat_image(current_image, &stat_data->current);
-    stat_image(next_image, &stat_data->next);
+    stat_image(previous_image, &stat_data->previous, quiet);
+    stat_image(current_image, &stat_data->current, quiet);
+    stat_image(next_image, &stat_data->next, quiet);
 
     return stat_data;
 }
@@ -597,7 +607,7 @@ gpointer stat_loaded_files(void)
 void reload_changed_files(gpointer * stat_data)
 {
     struct image_stat3 *before = (struct image_stat3 *) stat_data;
-    struct image_stat3 *now = stat_loaded_files();
+    struct image_stat3 *now = stat_loaded_files(TRUE);
 
     if (DIFFERENT(previous))
         forget_image(&previous_image);
@@ -680,15 +690,16 @@ gboolean menu_load(const gchar * filename)
     GlivImage *im;
     static gboolean loading = FALSE;
 
-    if (loading) {
+    if (loading || currently_loading()) {
         /* We are not reentrant. */
         return FALSE;
     }
 
     loading = TRUE;
 
-    if (currently_loading() || filename == current_image->node->data ||
-        check_direction(filename, -1) || check_direction(filename, 1)) {
+    if (current_image->node != NULL &&
+        (filename == current_image->node->data ||
+         check_direction(filename, -1) || check_direction(filename, 1))) {
         loading = FALSE;
         return FALSE;
     }
@@ -793,3 +804,12 @@ gboolean toggle_slide_show(void)
 
     return FALSE;
 }
+
+void new_images(gint nb_inserted)
+{
+    if (nb_inserted == 0)
+        return;
+
+    open_next_image(nb_inserted == 1);
+    do_later(G_PRIORITY_LOW, cond_rebuild_menus);
+}
