@@ -140,8 +140,7 @@ static void open_images(gchar ** paths, gint nb)
 {
     gint nb_inserted = insert_after_current(paths, nb, FALSE, FALSE);
 
-    if (nb_inserted > 0)
-        open_next_image(nb_inserted == 1);
+    new_images(nb_inserted);
 }
 
 static void load_selection(GSList * selection)
