@@ -39,7 +39,8 @@
 #endif
 
 static GHashTable *table = NULL;
-static gchar *config_file = NULL;
+static gchar *read_config_file = NULL;
+static gchar *write_config_file = NULL;
 
 /* The link between options and the rcfile. */
 static options_struct opts = {
@@ -64,20 +65,20 @@ static options_struct opts = {
     .thumbnails = TRUE,
     .resize_win = TRUE,
     .start_show = FALSE,
-    .no_center = FALSE,
     .confirm_quit = TRUE,
     .save_quit = TRUE,
     .recursive = FALSE,
     .transitions = TRUE,
     .keep_transfo = FALSE,
     .opengl_errors = FALSE,
+    .filtering = TRUE,
     .duration = 10,
     .fps = 100,
     .thumb_width = 128,
     .thumb_height = 64,
     .notice_time = 500,
-    .trans_steps = 20,
-    .trans_sleep = 10,
+    .trans_time = 500,
+    .initial_pos = POSITION_CENTER,
     .bg_col = {0, 0, 0, 0},
     .alpha1 = {0, 0x6666, 0x6666, 0x6666},
     .alpha2 = {0, 0x9999, 0x9999, 0x9999}
@@ -95,9 +96,9 @@ static option_struct option_names[] = {
 /* To fill the hash table and the configuration file. */
 { "full-screen",  &opts.fullscreen,   N_("Start in full screen mode"),      1 },
 { "maximize",     &opts.maximize,     N_("Maximize small images"),          1 },
-{ "scale-down",   &opts.scaledown,    N_("Scale down large images"),       1 },
+{ "scale-down",   &opts.scaledown,    N_("Scale down large images"),        1 },
 { "menu",         &opts.menu_bar,     N_("Display the menu bar"),           1 },
-{ "info",         &opts.status_bar,   N_("Display info about the image"),  1 },
+{ "info",         &opts.status_bar,   N_("Display info about the image"),   1 },
 { "scrollbars",   &opts.scrollbars,   N_("Display scrollbars"),             1 },
 { "zoom-pointer", &opts.zoom_pointer, N_("Zoom centered on pointer"),       1 },
 { "alpha-checks", &opts.alpha_checks, N_("Alpha checks in the background"), 1 },
@@ -111,13 +112,13 @@ static option_struct option_names[] = {
 { "thumbnails",   &opts.thumbnails,   N_("Show thumbnails in images menus"),1 },
 { "start_show",   &opts.start_show,   N_("Start with the slide show"),      1 },
 { "resize_win",   &opts.resize_win,   N_("Automatic window resizing"),      1 },
-{ "no_center",    &opts.no_center,    N_("No automatic image centering"),   1 },
 { "confirm_quit", &opts.confirm_quit, N_("Confirm before quitting"),        1 },
 { "save_quit",    &opts.save_quit,    N_("Save options when quitting"),     1 },
 { "transitions",  &opts.transitions,  N_("Enable transitions"),             1 },
 { "recursive",    &opts.recursive,    N_("Recursive directory traversal"),  1 },
 { "keep_transfo", &opts.keep_transfo, N_("Keep transfo between images"),    1 },
 { "opengl-errors",&opts.opengl_errors,N_("Check for OpenGL errors"),        1 },
+{ "filtering",    &opts.filtering,    N_("Enable OpenGL filtering"),        1 },
 { "fps",          &opts.fps,          N_("Maximum framerate"),              0 },
 { "delay",        &opts.delay,        N_("Delay before hiding the cursor"), 0 },
 { "history",      &opts.history_size, N_("History length"),                 0 },
@@ -125,8 +126,8 @@ static option_struct option_names[] = {
 { "thumb-width",  &opts.thumb_width,  N_("Thumbnail width"),                0 },
 { "thumb-height", &opts.thumb_height, N_("Thumbnail height"),               0 },
 { "notice-time",  &opts.notice_time,  N_("Last/First image notice time"),   0 },
-{ "trans-steps",  &opts.trans_steps,  N_("Steps count in a transition"),    0 },
-{ "trans-sleep",  &opts.trans_sleep,  N_("Time to wait between each step"), 0 },
+{ "trans-time",   &opts.trans_time,   N_("Transition duration"),            0 },
+{ "initial_pos",  &opts.initial_pos,  N_("Initial image position"),         0 },
 { "bg_col_red",   &opts.bg_col.red,   N_("background: red channel"),        0 },
 { "bg_col_green", &opts.bg_col.green, N_("background: green channel"),      0 },
 { "bg_col_blue",  &opts.bg_col.blue,  N_("background: blue channel"),       0 },
@@ -156,6 +157,7 @@ static const gchar *user_glivrc(void)
     static const gchar *filename = NULL;
 
     if (filename == NULL)
+        /* First time */
         filename = g_build_filename(g_get_home_dir(), ".glivrc", NULL);
 
     return filename;
@@ -281,8 +283,7 @@ static gboolean load_rc_file(const gchar * filename)
         return FALSE;
     }
 
-    while (feof(file) == 0) {
-        getdelim(&line, &nb, '\n', file);
+    while (getdelim(&line, &nb, '\n', file) >= 0) {
         if (g_str_equal(line, SEPARATOR_ACCELERATORS) ||
             g_str_equal(line, SEPARATOR_ACTIONS))
             break;
@@ -304,7 +305,8 @@ options_struct *load_rc(gboolean default_file, const gchar * filename)
 
     init_hash_table();
 
-    config_file = g_strdup(foreach_rcfile(load_rc_file, filename));
+    read_config_file = g_strdup(foreach_rcfile(load_rc_file, filename));
+    write_config_file = g_strdup(filename);
 
     g_hash_table_destroy(table);
 
@@ -382,9 +384,7 @@ static void load_actions_file(const gchar * filename)
         return;
     }
 
-    while (feof(file) == 0) {
-        getdelim(&line, &nb, '\n', file);
-
+    while (getdelim(&line, &nb, '\n', file) >= 0) {
         if (g_str_equal(line, SEPARATOR_ACCELERATORS) ||
             g_str_equal(line, SEPARATOR_ACTIONS))
             break;
@@ -398,7 +398,7 @@ static void load_actions_file(const gchar * filename)
 
 void load_actions(void)
 {
-    load_actions_file(config_file);
+    load_actions_file(read_config_file);
 }
 
 static void load_accelerators_file(const gchar * filename)
@@ -445,7 +445,7 @@ static void load_accelerators_file(const gchar * filename)
 
 void load_accelerators(void)
 {
-    load_accelerators_file(config_file);
+    load_accelerators_file(read_config_file);
 }
 
 
@@ -517,22 +517,38 @@ void save_rc(options_struct * opt)
 
 const gchar *get_read_config_file(void)
 {
-    return config_file;
+    return read_config_file;
+}
+
+/* Yes it's racy, but there's nothing wrong in this case */
+static gboolean can_write_config_file(const gchar * file)
+{
+    gchar *dirname;
+    gboolean res;
+
+    if (file == NULL)
+        return FALSE;
+
+    if (g_file_test(file, G_FILE_TEST_IS_DIR))
+        return FALSE;
+
+    if (g_file_test(file, G_FILE_TEST_EXISTS))
+        return !access(file, W_OK);
+
+    dirname = g_path_get_dirname(file);
+    res = g_file_test(dirname, G_FILE_TEST_IS_DIR) && !access(dirname, W_OK);
+    g_free(dirname);
+
+    return res;
 }
 
 const gchar *get_write_config_file(void)
 {
-    gchar *dirname;
-    const gchar *res = NULL;
+    if (can_write_config_file(write_config_file))
+        return write_config_file;
 
-    if (config_file != NULL && !access(config_file, W_OK))
-        return config_file;
+    if (can_write_config_file(user_glivrc()))
+        return user_glivrc();
 
-    dirname = g_path_get_dirname(user_glivrc());
-
-    if (!access(user_glivrc(), W_OK) || !access(dirname, W_OK))
-        res = user_glivrc();
-
-    g_free(dirname);
-    return res;
+    return NULL;
 }
