@@ -165,9 +165,11 @@ static GdkPixbuf *make_tile(texture_data * data)
     gint real_w, real_h;
 
     if (data->x == 0 && data->y == 0 &&
-        data->w == data->map->width && data->h == data->map->height)
+        data->w == data->map->width && data->h == data->map->height) {
         /* Special case, the image is OK for an OpenGL texture. */
+        g_object_ref(data->map->pixbuf);
         return data->map->pixbuf;
+    }
 
     if (data->w == rt->max_texture_size && data->h == rt->max_texture_size) {
         tile = data->tile_max_size;
@@ -207,7 +209,7 @@ static GdkPixbuf *make_tile(texture_data * data)
 static void make_texture(GlivImage * im, gint level, gint x, gint y,
                          gint w, gint h)
 {
-    guint texture0, texture1;
+    gint texture0, texture1;
     static GdkPixbuf *tile_max_size = NULL;
     texture_data *data;
     texture_map *map;
